@@ -2,6 +2,7 @@ from contextlib import closing
 from io import StringIO
 import platform
 import socket
+import uuid
 import time
 import sys
 import re
@@ -98,9 +99,11 @@ class Shell(transitions.Machine):
         credentials=(),
         ssh_keepalive=60,
         protocols=({"proto": "ssh", "port": 22}, {"proto": "telnet", "port": 23}),
+        mode="",
         auto_priv_mode=None,
         log_screen=False,
         log_file="",
+        strip_colors=True,
         debug=False,
         command_timeout=30,
         login_timeout=10,
@@ -155,26 +158,15 @@ class Shell(transitions.Machine):
         self.credentials_iterator = self.iter_credentials()
         self.proto_dict = {}
 
+        self.mode = mode
+        self.strip_colors = strip_colors
+
         self.prompt_hostname = ""
         self.prompt_str = ""  # This gets set in self.sync_prompt()
         # Detect a typical linux CLI prompt...
-        # self.linux_prompt_capture = '([^\r\n{0}]+)\s*$'.format(re.escape('$'))
-        self.linux_prompt_capture = "[\r\n]*[^\r\n{0}]+\s*{0}".format(re.escape("$"))
         # Build the template before detecting prompt
         self.base_prompt_regex = self.build_base_prompt_regex()
 
-        # Define regex capture groups for the prompts above...
-        # NOTE there are no prompts in these strings..
-        self.base_prompt_regex_capture = [
-            ":",
-            ":",
-            "(\S[^\n\r>]+?)\s*$",
-            self.linux_prompt_capture,
-            "(\S[^\n\r{0}]+?)\s*$".format(re.escape("#")),
-        ]
-        #'(\S[^\n\r>]+?)\s*>', self.linux_prompt_capture,
-        #'(\S[^\n\r{0}]+?)\s*{0}'.format(re.escape('#'))]
-
         self.matching_prompt_regex = ""
         self.matching_prompt_regex_index = -1
         self.matching_string = ""
@@ -490,10 +482,8 @@ class Shell(transitions.Machine):
             self.child.send(cmd)
 
         # Extend the list of cli_prompts if `prompts` was specified
-        # FIXME
-        # cli_prompts = self.base_prompt_regex
-        cli_prompts = self.build_base_prompt_regex()
-
+        self.build_base_prompt_regex()
+        cli_prompts = self.base_prompt_regex
         if prompts != ():
             assert isinstance(prompts, tuple)
             cli_prompts.extend(prompts)  # Add command-specific prompts here...
@@ -505,8 +495,15 @@ class Shell(transitions.Machine):
                     cmd
                 )
             )
+
         index = self.cexpect(cli_prompts, timeout=command_timeout)
 
+        if self.debug:
+            rich_print(
+                "    [bold blue]execute() received %s from cexpect()[/bold blue]"
+                % index
+            )
+
         # Handle sudo password prompt...
         if False and index == 3:  # TODO remove False (and maybe all this under it)
             if self.debug:
@@ -637,6 +634,9 @@ class Shell(transitions.Machine):
                     )
                 )
                 rich_print("")
+                rich_print("    [bold blue]debugs from str(self.child):[/bold blue]")
+                for line in str(self.child).splitlines():
+                    rich_print("        [bold yellow]{0}[/bold yellow]".format(line))
 
         except px.exceptions.EOF:
             if self.debug:
@@ -760,7 +760,10 @@ class Shell(transitions.Machine):
 
     @property
     def response(self):
-        return self.child.before
+        if self.strip_colors:
+            return self.strip_text_colors(self.child.before)
+        else:
+            return self.child.before
 
     def exit(self):
         self.child.close()
@@ -890,7 +893,6 @@ class Shell(transitions.Machine):
                 # We don't need to attempt any more logins if we have
                 #     a priv prompt
                 self.login_attempts = 0
-                print("BARMEBAR")
                 finished = True
 
             else:
@@ -912,6 +914,17 @@ class Shell(transitions.Machine):
                 )
             )
 
+        if self.mode=="linux":
+            self.change_linux_prompt()
+
+            # Catch up on missed prompts...
+            finished = False
+            while not finished:
+                try:
+                    self.child.expect(self.base_prompt_regex, timeout=1)
+                except:
+                    finished = True
+
         if self.debug:
             rich_print(
                 "    [bold blue]detect_prompt() using prompt detection heuristics[/bold blue]"
@@ -1108,8 +1121,21 @@ class Shell(transitions.Machine):
     def build_base_prompt_regex(self):
         """Assign self.base_prompt_regex with the latest prompt info"""
 
-        self.linux_prompt = r"[\n\r]+{0}[^{1}]*?{1}\s*".format(
-            re.escape(self.prompt_hostname), re.escape("$")
+        ####### WARNING #######################################################
+        #   Getting this cisco unpriv prompt to work was extremely hard...
+        #   there is a bug somewhere that ignores the trailing ">" and
+        #   instead matches a space after the hostname.  I forced the
+        #   regex to match properly by stripping off the last character of
+        #   the hostname and used a negated special character class: [^\n]+?
+        #######################################################################
+        cisco_unpriv_prompt_str = r"[\r\n]+{0}[^\n]+?{1}".format(
+            re.escape(self.prompt_hostname[:-1]), r">"
+        )
+        linux_prompt_str = r"[\r\n]+{0}[^\n]+?{1}".format(
+            re.escape(self.prompt_hostname[:-1]), re.escape("$")
+        )
+        cisco_priv_prompt_str = r"[\r\n]+{0}[^\n]+?{1}".format(
+            re.escape(self.prompt_hostname[:-1]), re.escape("#")
         )
 
         self.base_prompt_regex = [
@@ -1119,15 +1145,25 @@ class Shell(transitions.Machine):
             r"no\s+matching\s+key\s+exchange\s+method\s+found\..*$",
             r"sername:",
             r"[Pp]assword[^\r\n]{0,20}?:",
-            # r'[\n\r]+{}[^\n\r>]*?>\s*'.format(self.prompt_str),
-            r"[\n\r]+{0}.*?>\s*".format(re.escape(self.prompt_hostname)),
-            # linux_prompt, r'[\n\r]+{0}[^\n\r#]*?{1}\s*'.format(
-            self.linux_prompt,
-            r"[\n\r]+{0}.*?{1}\s*".format(
-                re.escape(self.prompt_hostname), re.escape("#")
-            ),
+            # Cisco unpriv prompt...
+            cisco_unpriv_prompt_str,
+            # linux prompt
+            linux_prompt_str,
+            # Cisco priv or linux root prompt...
+            cisco_priv_prompt_str,
         ]
 
+        # Remove some prompt matches after successful login
+        if self.state in set({
+            "LOGIN_SUCCESS_UNPRIV",
+            "LOGIN_SUCCESS_PRIV",
+            "LOGIN_TIMEOUT",
+            "LOGIN_COMPLETE",
+            }):
+            for index in [0, 1, 2, 3]:
+                self.base_prompt_regex[index] = str(uuid.uuid4())
+
+
         if self.debug:
             # Expand all base_prompt_regex terms...
             rich_print(
@@ -1144,6 +1180,25 @@ class Shell(transitions.Machine):
         assert len(self.base_prompt_regex) == BASE_PROMPT_REGEX_LENGTH
         return self.base_prompt_regex
 
+    def change_linux_prompt(self):
+        if self.debug:
+            rich_print("")
+            rich_print("    [bold cyan]in change_linux_prompt()[/bold cyan]")
+        self.execute("precmd_functions=()")
+        self.execute("export PS1='linux>'")
+
+    def strip_text_colors(self, ascii_text=""):
+        """This function only works with string inputs, byte inputs fail"""
+        # https://stackoverflow.com/a/14693789/667301...
+        assert not isinstance(ascii_text, bytes)
+        assert isinstance(ascii_text, str)
+        ansi_escape_8bit = re.compile(
+            br"(?:\x1B[@-Z\\-_]|[\x80-\x9A\x9C-\x9F]|(?:\x1B\[|\x9B)[0-?]*[ -/]*[@-~])"
+        )
+        result = ansi_escape_8bit.sub(b"", bytes(ascii_text, self.encoding))
+        return result.decode(self.encoding)
+
+
     def after_SELECT_TCP_PROTOCOL_cb(self):
         """Attempt to make a raw TCP connection to the protocol's TCP port"""
 
@@ -1651,14 +1706,6 @@ class Shell(transitions.Machine):
                 )
             )
 
-        # OLD FIXME
-        # if self.debug:
-        #    rich_print("   [bold blue]after_LOGIN_SUCCESS_UNPRIV_cb() - csendline("") <- blank line[/bold blue]}")
-        # self.csendline('')
-
-        # index = self.cexpect(self.base_prompt_regex,
-        #    timeout=self.command_timeout)
-
         if self.debug:
             rich_print(
                 "    [bold blue]after_LOGIN_SUCCESS_UNPRIV_cb() calling detect_prompt()[/bold blue]"
